@@ -27,6 +27,7 @@ from abc import ABCMeta, abstractmethod
 from typing import Optional, Iterable, List
 
 import numpy as np
+import pyannote.core.segment
 from pyannote.core import Segment
 from sortedcontainers import SortedSet
 from typing_extensions import Literal
@@ -325,12 +326,16 @@ class StatisticalContinuumSampler(AbstractContinuumSampler):
                 nb_units = max(1, nb_units)
             for _ in range(nb_units):
                 gap = np.random.normal(self._avg_gap, self._std_gap)
-                length = abs(np.random.normal(self._avg_unit_duration, self._std_unit_duration))
-                while length == 0:
-                    length = abs(np.random.normal(self._avg_unit_duration, self._std_unit_duration))
-                category = np.random.choice(self._categories, p=self._categories_weight)
                 start = last_point + gap
-                end = start + length
+
+                end = start + abs(np.random.normal(self._avg_unit_duration, self._std_unit_duration))
+                # Segments shorter than segment precision are illegal for pyannote
+                while end - start < pyannote.core.segment.SEGMENT_PRECISION:
+                    end = start + abs(np.random.normal(self._avg_unit_duration, self._std_unit_duration))
+
+                category = np.random.choice(self._categories, p=self._categories_weight)
+
                 new_continnum.add(annotator, Segment(start, end), category)
+
                 last_point = end
         return new_continnum
